@@ -125,12 +125,24 @@ def basil(wsp, output_wsp=None, prefit=True):
     # Pick up extra BASIL options
     extra_options = dict(wsp.ifnone("basil_options", {}))
 
+    # If we only have one volume, set a nominal noise prior as it is not possible to
+    # estimate from the data
+    if wsp.asldata.nvols / wsp.asldata.ntc == 1:
+        wsp.log.write(" - Restricting noise prior as only one ASL volume\n")
+        extra_options["prior-noise-stddev"] = 1.0
+    
+    # Mask - for compatibility check in workspace and in rois
+    if wsp.rois is not None and wsp.rois.mask is not None:
+        mask = wsp.rois.mask
+    else:
+        mask = wsp.mask
+
     if prefit and max(wsp.asldata.rpts) > 1:
         # Initial BASIL run on mean data
         wsp.log.write(" - Doing initial fit on mean at each TI\n\n")
         init_wsp = output_wsp.sub("init")
         main_wsp = output_wsp.sub("main")
-        basil_fit(wsp, wsp.asldata.mean_across_repeats(), mask=wsp.rois.mask, output_wsp=init_wsp, **extra_options)
+        basil_fit(wsp, wsp.asldata.mean_across_repeats(), mask=mask, output_wsp=init_wsp, **extra_options)
         extra_options["continue-from-mvn"] = output_wsp.init.finalstep.finalMVN
         main_wsp.initmvn = extra_options["continue-from-mvn"]
     else:
@@ -138,7 +150,7 @@ def basil(wsp, output_wsp=None, prefit=True):
 
     # Main run on full ASL data
     wsp.log.write("\n - Doing fit on full ASL data\n\n")
-    basil_fit(wsp, wsp.asldata, mask=wsp.rois.mask, output_wsp=main_wsp, **extra_options)
+    basil_fit(wsp, wsp.asldata, mask=mask, output_wsp=main_wsp, **extra_options)
     output_wsp.finalstep = main_wsp.finalstep
 
 def basil_fit(wsp, asldata, mask=None, output_wsp=None, **kwargs):
@@ -173,6 +185,12 @@ def basil_fit(wsp, asldata, mask=None, output_wsp=None, **kwargs):
                           fabber_corelib=wsp.fabber_corelib, fabber_libs=wsp.fabber_libs,
                           fabber_coreexe=wsp.fabber_coreexe, fabber_exes=wsp.fabber_exes)
         for key, value in result.items():
+            if key == "modelfit":
+                # Treat model fit specially - make it an AslImage and also output a mean
+                # across repeats version for comparison
+                value = output_wsp.asldata_diff.derived(value.data)
+                modelfit_mean = value.mean_across_repeats()
+                setattr(step_wsp, "modelfit_mean", modelfit_mean)
             setattr(step_wsp, key, value)
 
         if step_wsp.logfile is not None and step_wsp.savedir is not None:
@@ -216,6 +234,7 @@ def basil_steps(wsp, asldata, mask=None, **kwargs):
         "save-mvn" : True,
         "save-std" : True,
         "save-model-fit" : True,
+        "save-residuals" : wsp.ifnone("output_residuals", False),
     }
 
     if mask is not None:
@@ -252,7 +271,7 @@ def basil_steps(wsp, asldata, mask=None, **kwargs):
                 datamax = np.amax(wsp.diffdata_mean.data, 3)
             else:
                 datamax = wsp.diffdata_mean.data
-            brain_mag = np.mean(datamax.data[wsp.rois.mask.data != 0])
+            brain_mag = np.mean(datamax.data[mask.data != 0])
             # this will correspond to whole brain CBF (roughly) - about 0.5 of GM
             noisesd = math.sqrt(brain_mag * 2 / snr)
         else:
@@ -264,7 +283,7 @@ def basil_steps(wsp, asldata, mask=None, **kwargs):
     options.update(kwargs)
 
     # Additional optional workspace arguments
-    for attr in ("t1", "t1b", "bat", "FA", "mask", "pwm", "pgm", "batsd"):
+    for attr in ("t1", "t1b", "bat", "FA", "pwm", "pgm", "batsd"):
         value = getattr(wsp, attr)
         if value is not None:
             options[attr] = value
@@ -334,9 +353,16 @@ def basil_steps(wsp, asldata, mask=None, **kwargs):
         options["continue-from-mvn"] = wsp.initmvn
 
     # T1 image prior
-    if wsp.t1im:
+    if wsp.t1im is not None:
         spriors = _add_prior(options, spriors, "T_1", type="I", image=wsp.t1im)
 
+    # BAT image prior
+    if wsp.batim is not None:
+        # With a BAT image prior we must include BAT even if we are not inferring it
+        # (in this case the image prior will be treated as ground truth)
+        spriors = _add_prior(options, spriors, "delttiss", type="I", image=wsp.batim)
+        options["incbat"] = True
+
     steps = []
     components = ""
 
@@ -408,7 +434,7 @@ def basil_steps(wsp, asldata, mask=None, **kwargs):
 
         if steps:
             # Add initialisaiton step for PV correction - ONLY if we have something to init from
-            steps.append(PvcInitStep({"data" : asldata, "mask" : wsp.rois.mask, "pgm" : pgm, "pwm" : pwm}, "PVC initialisation"))
+            steps.append(PvcInitStep({"data" : asldata, "mask" : mask, "pgm" : pgm, "pwm" : pwm}, "PVC initialisation"))
 
     ### --- SPATIAL MODULE ---
     if wsp.spatial:
@@ -494,7 +520,7 @@ def basil_steps_multite(wsp, asldata, mask=None, **kwargs):
     options.update(kwargs)
 
     # Additional optional workspace arguments
-    for attr in ("t1", "t1b", "t2", "t2b", "mask"):
+    for attr in ("t1", "t1b", "t2", "t2b"):
         value = getattr(wsp, attr)
         if value is not None:
             if attr.startswith("t2"):
@@ -541,6 +567,13 @@ def basil_steps_multite(wsp, asldata, mask=None, **kwargs):
     if wsp.t1im:
         spriors = _add_prior(options, spriors, "T_1", type="I", image=wsp.t1im)
 
+    # BAT image prior
+    if wsp.batim is not None:
+        # With a BAT image prior we must include BAT even if we are not inferring it
+        # (in this case the image prior will be treated as ground truth)
+        spriors = _add_prior(options, spriors, "delttiss", type="I", image=wsp.batim)
+        options["incbat"] = True
+
     steps = []
     components = ""
 
@@ -628,7 +661,6 @@ class PvcInitStep(Step):
         for PVC parameters
         """
         log.write("Initialising partial volume correction...\n")
-        mask = self.options["mask"]
         # set the inital GM amd WM values using a simple PV correction
         wm_cbf_ratio = 0.4
 
@@ -647,6 +679,10 @@ class PvcInitStep(Step):
         gmcbf_init = Image(gmcbf_init, header=mvn.header)
         wmcbf_init = Image(wmcbf_init, header=mvn.header)
 
+        # HACK: This seems to be required to get the fslpy decorators to write
+        # the temporary file correctly
+        mask = Image(self.options["mask"].data, header=self.options["mask"].header)
+
         # load these into the MVN
         mvn = prev_output["finalMVN"]
         from .wrappers import mvntool
@@ -695,6 +731,7 @@ class BasilOptions(OptionCategory):
 
         group = IgnorableOptionGroup(parser, "Special options", ignore=self.ignore)
         group.add_option("--t1im", help="Voxelwise T1 tissue estimates", type="image")
+        group.add_option("--batim", "--attim", help="Voxelwise BAT (ATT) estimates in seconds", type="image")
         groups.append(group)
 
         return groups
