@@ -149,7 +149,7 @@ def get_m0_voxelwise(wsp):
     wsp.log.write(" - Calibration gain: %f\n" % gain)
 
     # Calculate M0 value
-    m0 = np.copy(wsp.calib.data).astype(np.float) * gain
+    m0 = np.copy(wsp.calib.data).astype(np.float32) * gain
 
     shorttr = 1
     if wsp.tr is not None and wsp.tr < 5:
@@ -164,15 +164,18 @@ def get_m0_voxelwise(wsp):
     wsp.log.write(" - Using partition coefficient: %f\n" % pct)
     m0 /= pct
 
+    edgecorr = wsp.ifnone("calib_edgecorr", True)
     if wsp.rois is not None and wsp.rois.mask is not None:
-        if wsp.ifnone("calib_edgecorr", True):
+        if edgecorr:
             wsp.log.write(" - Doing edge correction\n")
             m0 = _edge_correct(m0, wsp.rois.mask)
         wsp.log.write(" - Masking M0 image")
         m0[wsp.rois.mask.data == 0] = 0
+        wsp.log.write(" - mean in mask: %f\n" % np.mean(m0[wsp.rois.mask.data > 0]))
+    else:
+        wsp.log.write(" - Mean M0 (no mask): %f\n" % np.mean(m0))
 
     m0img = Image(m0, header=wsp.calib.header)
-    wsp.log.write(" - Mean M0: %f\n" % np.mean(m0))
 
     # Reporting
     page = wsp.report.page("m0")
@@ -191,7 +194,7 @@ def get_m0_voxelwise(wsp):
 
     if wsp.rois is not None and wsp.rois.mask is not None:
         table.append(["Mean M0 (within mask)", "%.3g" % np.mean(m0[wsp.rois.mask.data > 0])])
-        table.append(["Edge correction", "Enabled" if wsp.edgecorr else "Disabled"])
+        table.append(["Edge correction", "Enabled" if edgecorr else "Disabled"])
     else:
         table.append(["Mean M0", "%.3g" % np.mean(m0)])
     page.table(table)
@@ -287,7 +290,7 @@ def get_m0_wholebrain(wsp):
         wsp.log.write(" - Using sensitivity image: %s\n" % wsp.sens.name)
         calib_data /= wsp.sens.data
 
-    m0 = np.zeros(calib_data.shape, dtype=np.float)
+    m0 = np.zeros(calib_data.shape, dtype=np.float32)
     for tiss_type in ("wm", "gm", "csf"):
         pve_struc = getattr(wsp.structural, "%s_pv" % tiss_type)
         wsp.log.write(" - Transforming %s tissue PVE into ASL space\n" % tiss_type)
@@ -414,7 +417,7 @@ def get_m0_refregion(wsp, mode="longtr"):
     wsp.log.write(" - T1r: %f; T2r: %f; T2b: %f; Part co-eff: %f\n" % (t1r, t2r, t2b, pcr))
 
     # Check the data and masks
-    calib_data = np.copy(wsp.calib.data).astype(np.float)
+    calib_data = np.copy(wsp.calib.data).astype(np.float32)
     wsp.calibration.calib_img = wsp.calib
     if calib_data.ndim == 4:
         wsp.log.write(" - Taking mean across calibration images\n")
@@ -427,7 +430,7 @@ def get_m0_refregion(wsp, mode="longtr"):
 
     if wsp.refmask is not None:
         wsp.log.write(" - Using supplied reference tissue mask: %s" % wsp.refmask.name)
-        wsp.calibration.refmask = Image(wsp.refmask.data.astype(np.int), header=wsp.refmask.header)
+        wsp.calibration.refmask = Image(wsp.refmask.data.astype(np.int32), header=wsp.refmask.header)
         if wsp.calib_aslreg:
             wsp.log.write(" (Aligned to ASL image already)\n")
             wsp.calibration.refmask_trans = wsp.calibration.refmask
@@ -607,8 +610,8 @@ def get_tissrefmask(wsp):
         atlases = AtlasRegistry()
         atlases.rescanAtlases()
         atlas = atlases.loadAtlas("harvardoxford-subcortical", loadSummary=False, resolution=2)
-        ventricles = ((atlas.data[..., 2] + atlas.data[..., 13]) > 0.1).astype(np.int)
-        wsp.calibration.ventricles = Image(scipy.ndimage.binary_erosion(ventricles, structure=np.ones([3, 3, 3]), border_value=1).astype(np.int), header=atlas.header)
+        ventricles = ((atlas.data[..., 2] + atlas.data[..., 13]) > 0.1).astype(np.int32)
+        wsp.calibration.ventricles = Image(scipy.ndimage.binary_erosion(ventricles, structure=np.ones([3, 3, 3]), border_value=1).astype(np.int32), header=atlas.header)
         std_img = Image(os.path.join(os.environ["FSLDIR"], "data", "standard", 'MNI152_T1_2mm_brain'))
         page.image("ventricles_std", LightboxImage(wsp.calibration.ventricles, bgimage=std_img))
 
@@ -640,7 +643,7 @@ def get_tissrefmask(wsp):
 
     # Threshold reference mask conservatively to select only reference tissue
     wsp.log.write(" - Thresholding reference mask\n")
-    wsp.calibration.refmask = Image((wsp.calibration.refpve_calib.data > 0.9).astype(np.int), header=wsp.calibration.refpve_calib.header)
+    wsp.calibration.refmask = Image((wsp.calibration.refpve_calib.data > 0.9).astype(np.int32), header=wsp.calibration.refpve_calib.header)
 
     page.text("Reference Mask (thresholded at 0.9")
     page.image("refmask", LightboxImage(wsp.calibration.refmask, bgimage=wsp.calib))
