@@ -44,7 +44,7 @@ from oxasl.wrappers import epi_reg, fnirtfileutils
 
 class DistcorrOptions(OptionCategory):
     """
-    OptionCategory which contains options for distortion correction
+    OptionCategory which contains options for corrections of the input data
     """
 
     def __init__(self, **kwargs):
@@ -66,7 +66,7 @@ class DistcorrOptions(OptionCategory):
         g = IgnorableOptionGroup(parser, "General distortion correction options")
         g.add_option("--echospacing", help="Effective EPI echo spacing (sometimes called dwell time) - in seconds", type=float)
         g.add_option("--pedir", help="Phase encoding direction, dir = x/y/z/-x/-y/-z")
-        g.add_option("--gdcwarp", help="Additional warp image for gradient distortion correction - will be combined with fieldmap or TOPUP distortion correction", type="image")
+        g.add_option("--gdc-warp", "--gdcwarp", help="Additional warp image for gradient distortion correction - will be combined with fieldmap or TOPUP distortion correction", type="image")
         ret.append(g)
 
         g = IgnorableOptionGroup(parser, "Sensitivity correction")
@@ -77,9 +77,13 @@ class DistcorrOptions(OptionCategory):
         g.add_option("--senscorr-off", help="Do not apply any sensitivity correction", action="store_true", default=False)
         ret.append(g)
 
-        g = IgnorableOptionGroup(parser, "Partial volume correction")
-        g.add_option("--pvcorr", help="Apply partial volume correction", action="store_true", default=False)
-        g.add_option("--surf-pvcorr", help="Apply SURFACE PVEc, not mutually exclusive with --pvcorr", action="store_true", default=False)
+        g = IgnorableOptionGroup(parser, "Partial volume correction (PVEc)")
+        g.add_option("--pvcorr", help="Apply PVEc using FAST estimates taken from --fslanat dir", action="store_true", default=False)
+        g.add_option("--surf-pvcorr", help="Apply PVEc using surface PV estimates. Requires --fslanat and --fsdir, not mutually exclusive with --pvcorr)", action="store_true", default=False)
+        g.add_option("--fsdir", help="Path to FreeSurfer subject directory for --surf-pvcorr", default=None)
+        g.add_option('--cores', help="Number of processor cores to use for --surf-pvcorr", type=int)
+        g.add_option("--pvgm", help="GM PV estimates in ASL space (apply PVEc only, don't estimate PVs)", type="image", default=None)
+        g.add_option("--pvwm", help="As above, WM PV estimates in ASL space", type="image", default=None)
         ret.append(g)
 
         return ret
@@ -97,15 +101,17 @@ def single_volume(wsp, img, moco=True, discard_first=True):
         if img.ndim == 4:
             if discard_first and img.shape[3] > 1:
                 wsp.log.write("   - Removing first volume to ensure data is in steady state\n")
-                img = Image(img.data[..., :-1], header=img.header)
+                img = Image(img.data[..., 1:], header=img.header)
 
-            if moco and img.shape[3] > 1:
+            # Note that image may now be 3d if we have discarded a volume
+            if moco and img.ndim == 4 and img.shape[3] > 1:
                 if moco:
                     wsp.log.write("   - Motion correcting\n")
                     img = fsl.mcflirt(img, out=fsl.LOAD, log=wsp.fsllog)["out"]
 
-            wsp.log.write("   - Taking mean across time axis\n")
-            img = Image(np.mean(img.data, axis=-1), header=img.header)
+            if img.ndim == 4:
+                wsp.log.write("   - Taking mean across time axis\n")
+                img = Image(np.mean(img.data, axis=-1), header=img.header)
 
         return img
     else:
@@ -154,7 +160,7 @@ def get_cblip_correction(wsp):
         "y"  : 1, "-y" : 1,
         "z"  : 2, "-z" : 2,
     }
-    my_topup_params = np.array(topup_params[wsp.pedir], dtype=np.float)
+    my_topup_params = np.array(topup_params[wsp.pedir], dtype=np.float32)
     dimsize = wsp.asldata.shape[dim_idx[wsp.pedir]]
     my_topup_params[:, 3] = wsp.echospacing * (dimsize - 1)
     wsp.topup.params = my_topup_params
@@ -233,7 +239,11 @@ def get_fieldmap_correction(wsp):
     else:
         result = epi_reg(epi=wsp.asldata.perf_weighted(), t1=wsp.structural.struc, t1brain=wsp.structural.brain, out=fsl.LOAD, wmseg=wsp.structural.wm_seg, log=wsp.fsllog, **epi_reg_opts)
 
-    wsp.fieldmap.warp_struc = result["out_warp"]
+    # Occasionally we end up with NaN in the output of epi_reg and this will ruin the entire distcorr warp.
+    # So remove any NaN values and replace with zero.
+    warp_struc = result["out_warp"]
+    wsp.fieldmap.warp_struc = Image(np.nan_to_num(warp_struc.data, nan=0), header=warp_struc.header)
+
     wsp.fieldmap.asl2struc = result["out"]
     wsp.fieldmap.struc2asl = np.linalg.inv(wsp.fieldmap.asl2struc)
 
@@ -384,14 +394,14 @@ def get_sensitivity_correction(wsp):
         wsp.log.write(" - Sensitivity image calculated from calibration actual and reference images\n")
         cref_data = np.copy(wsp.cref.data)
         cref_data[cref_data == 0] = 1
-        sensitivity = Image(wsp.cact.data.astype(np.float) / cref_data, header=wsp.calib.header)
+        sensitivity = Image(wsp.cact.data.astype(np.float32) / cref_data, header=wsp.calib.header)
     elif wsp.calib is not None and wsp.cref is not None:
         if wsp.ifnone("mode", "longtr") != "longtr":
             raise ValueError("Calibration reference image specified but calibration image was not in longtr mode - need to provided additional calibration image using the ASL coil")
         wsp.log.write(" - Sensitivity image calculated from calibration and reference images\n")
         cref_data = np.copy(wsp.cref.data)
         cref_data[cref_data == 0] = 1
-        sensitivity = Image(wsp.calib.data.astype(np.float) / cref_data, header=wsp.calib.header)
+        sensitivity = Image(wsp.calib.data.astype(np.float32) / cref_data, header=wsp.calib.header)
     elif wsp.senscorr_auto and wsp.structural.bias is not None:
         struc.segment(wsp)
         wsp.log.write(" - Sensitivity image calculated from bias field\n")
@@ -452,10 +462,14 @@ def apply_corrections(wsp):
 
     wsp.corrected.asldata = wsp.input.asldata
     wsp.corrected.nativeref = wsp.input.asldata.mean()
-    wsp.corrected.calib = single_volume(wsp, wsp.input.calib)
-    wsp.corrected.cref = single_volume(wsp, wsp.input.cref)
-    wsp.corrected.cact = single_volume(wsp, wsp.input.cact)
-    wsp.corrected.cblip = single_volume(wsp, wsp.input.cblip)
+    wsp.corrected.calib_preproc = single_volume(wsp, wsp.input.calib)
+    wsp.corrected.cref_preproc = single_volume(wsp, wsp.input.cref)
+    wsp.corrected.cact_preproc = single_volume(wsp, wsp.input.cact)
+    wsp.corrected.cblip_preproc = single_volume(wsp, wsp.input.cblip)
+    wsp.corrected.calib = wsp.corrected.calib_preproc
+    wsp.corrected.cref = wsp.corrected.cref_preproc
+    wsp.corrected.cact = wsp.corrected.cact_preproc
+    wsp.corrected.cblip = wsp.corrected.cblip_preproc
 
     wsp.log.write(" - Data transformations\n")
     if wsp.moco is not None:
@@ -570,9 +584,9 @@ def correct_img(wsp, img, linear_mat):
     else:
         img = reg.transform(wsp, img, trans=linear_mat, ref=wsp.nativeref)
 
-    if wsp.corrected.total_jacobian is not None:
+    if wsp.corrected.jacobian is not None:
         wsp.log.write("   - Correcting for local volume scaling using Jacobian\n")
-        jdata = wsp.corrected.total_jacobian.data
+        jdata = wsp.corrected.jacobian.data
         if img.data.ndim == 4:
             # Required to make broadcasting work
             jdata = jdata[..., np.newaxis]
