@@ -16,7 +16,7 @@ import traceback
 
 import numpy as np
 
-from fsl.data.image import Image
+from fsl.data.image import Image, defaultExt
 import fsl.wrappers as fsl
 
 from oxasl import __version__, Workspace, struc, brain
@@ -99,7 +99,7 @@ def get_motion_params(mat):
             mat[2, 1] - mat[1, 2],
             mat[0, 2] - mat[2, 0],
             mat[1, 0] - mat[0, 1],
-        ], dtype=np.float)
+        ], dtype=np.float32)
 
         # Rotation angle - note that we need to check the sign
         trace = np.trace(mat[:3, :3])
@@ -122,12 +122,43 @@ def reg_asl2calib(wsp):
     Note that this might already have been done as part of motion correction
     """
     init(wsp)
-    if wsp.calib is not None and wsp.reg.asl2calib is None:
+    if wsp.calib is not None and wsp.reg.asl2calib is None and not wsp.calib_aslreg:
         get_regfrom(wsp)
         wsp.log.write("Registering calibration image to ASL image\n")
-        _, wsp.reg.asl2calib = reg_flirt(wsp, wsp.reg.regfrom, wsp.calib)
+        _, wsp.reg.asl2calib = reg_flirt(wsp, wsp.reg.regfrom, wsp.corrected.calib_preproc)
         wsp.reg.calib2asl = np.linalg.inv(wsp.reg.asl2calib)
 
+    if wsp.reg.asl2calib is not None:
+        wsp.log.write(" - ASL->Calibration transform\n")
+        wsp.log.write(str(wsp.reg.asl2calib) + "\n")
+        wsp.log.write(" - Calibration->ASL transform\n")
+        wsp.log.write(str(wsp.reg.calib2asl) + "\n")
+
+def reg_asl2custom(wsp):
+    """
+    Register custom output image to ASL space, via structural. 
+
+    If no output_custom_mat (struc -> custom) has been provided, then
+    FLIRT will be used to generate this. The overall transformation
+    is the concatenation of asl2struc and struc2custom. 
+    """
+
+    if wsp.output_custom:
+        init(wsp)
+        setattr(wsp.reg, 'customref', Image(wsp.output_custom))
+
+        if wsp.output_custom_mat is not None:
+            struc2custom = np.loadtxt(wsp.output_custom_mat)
+
+        else:  
+            wsp.log.write("Registering calibration image to ASL image\n")
+            _, custom2struc = reg_flirt(wsp, wsp.reg.customref,
+                wsp.structural.struc)
+            struc2custom = np.linalg.inv(custom2struc)
+
+        wsp.reg.asl2custom = struc2custom @ wsp.reg.asl2struc
+        wsp.reg.custom2asl = np.linalg.inv(wsp.reg.asl2custom)
+
 def reg_asl2struc(wsp, flirt=True, bbr=False, name="initial"):
     """
     Registration of ASL images to structural image
@@ -152,13 +183,24 @@ def reg_asl2struc(wsp, flirt=True, bbr=False, name="initial"):
     struc.init(wsp)
     if wsp.structural.struc is not None:
         get_regfrom(wsp)
-        wsp.log.write("\nRegistering ASL data to structural data\n")
-        if flirt:
-            wsp.reg.regto, wsp.reg.asl2struc = reg_flirt(wsp, wsp.reg.regfrom, wsp.structural.brain, wsp.reg.asl2struc)
-        if bbr:
-            wsp.reg.regto, wsp.reg.asl2struc = reg_bbr(wsp)
+        if wsp.struc2asl is not None or wsp.asl2struc is not None:
+            wsp.log.write("\nASL->Structural registration provided by user\n")
 
-        wsp.reg.struc2asl = np.linalg.inv(wsp.reg.asl2struc)
+            wsp.reg.struc2asl = wsp.struc2asl
+            wsp.reg.asl2struc = wsp.asl2struc
+            if wsp.reg.asl2struc is None:
+                wsp.reg.asl2struc = np.linalg.inv(wsp.reg.struc2asl)
+            if wsp.reg.struc2asl is None:
+                wsp.reg.struc2asl = np.linalg.inv(wsp.reg.asl2struc)
+            wsp.reg.regto = asl2struc(wsp, wsp.reg.regfrom)
+        else:
+            wsp.log.write("\nRegistering ASL data to structural data\n")
+            if flirt:
+                wsp.reg.regto, wsp.reg.asl2struc = reg_flirt(wsp, wsp.reg.regfrom, wsp.structural.brain, wsp.reg.asl2struc)
+            if bbr:
+                wsp.reg.regto, wsp.reg.asl2struc = reg_bbr(wsp)
+
+            wsp.reg.struc2asl = np.linalg.inv(wsp.reg.asl2struc)
 
         wsp.log.write(" - ASL->Structural transform\n")
         wsp.log.write(str(wsp.reg.asl2struc) + "\n")
@@ -248,6 +290,13 @@ def struc2std(wsp, img, **kwargs):
     ref = Image(os.path.join(os.environ["FSLDIR"], "data/standard/MNI152_T1_2mm_brain"))
     return transform(wsp, img, wsp.reg.struc2std, ref, **kwargs)
 
+def asl2custom(wsp, img, **kwargs):
+    """
+    Transform an image from ASL space to custom output (via structural)
+    """
+    init(wsp)
+    return transform(wsp, img, wsp.reg.asl2custom, wsp.reg.customref, **kwargs)
+
 def struc2asl(wsp, img, **kwargs):
     """
     Convert an image from structural to ASL space
@@ -278,7 +327,10 @@ def calib2asl(wsp, img, **kwargs):
     :return: Transformed Image object in ASL (native) space
     """
     init(wsp)
-    return transform(wsp, img, wsp.reg.calib2asl, wsp.nativeref, **kwargs)
+    if wsp.calib_aslreg:
+        return img
+    else:
+        return transform(wsp, img, wsp.reg.calib2asl, wsp.nativeref, **kwargs)
 
 def asl2calib(wsp, img, **kwargs):
     """
@@ -292,7 +344,20 @@ def asl2calib(wsp, img, **kwargs):
     init(wsp)
     return transform(wsp, img, wsp.reg.asl2calib, wsp.structural.struc, **kwargs)
 
-def transform(wsp, img, trans, ref, use_flirt=False, interp="trilinear", paddingsize=1, premat=None, mask=False, mask_thresh=0.5):
+def std2asl(wsp, img, **kwargs):
+    """
+    Transform an image from standard (MNI) space to native (ASL) space
+    """
+    return transform(wsp, img, wsp.reg.std2struc, wsp.nativeref, postmat=wsp.reg.struc2asl, **kwargs)
+
+def asl2std(wsp, img, **kwargs):
+    """
+    Transform an image from  native (ASL) space to standard (MNI) space
+    """
+    stdref = Image(os.path.join(os.environ["FSLDIR"], "data/standard/MNI152_T1_2mm_brain"))
+    return transform(wsp, img, wsp.reg.struc2std, stdref, premat=wsp.reg.asl2struc, **kwargs)
+
+def transform(wsp, img, trans, ref, use_flirt=False, interp="trilinear", paddingsize=1, premat=None, postmat=None, mask=False, mask_thresh=0.5):
     """
     Transform an image
 
@@ -319,15 +384,15 @@ def transform(wsp, img, trans, ref, use_flirt=False, interp="trilinear", padding
         ret = fsl.applyxfm(img, ref, trans, out=fsl.LOAD, interp=interp, paddingsize=paddingsize, log=wsp.fsllog)["out"]
     else:
         if have_warp:
-            kwargs = {"warp" : trans, "premat" : premat, "rel" : True}
-        elif premat:
-            raise ValueError("Can't set a pre-transformation matrix unless using a warp")
+            kwargs = {"warp" : trans, "premat" : premat, "rel" : True, "postmat" : postmat}
+        elif premat is not None or postmat is not None:
+            raise ValueError("Can't set a pre/post transformation matrix unless using a warp")
         else:
             kwargs = {"premat" : trans}
         ret = fsl.applywarp(img, ref, out=fsl.LOAD, interp=interp, paddingsize=paddingsize, super=True, superlevel="a", log=wsp.fsllog, **kwargs)["out"]
     if mask:
         # Binarise mask images
-        ret = Image((ret.data > mask_thresh).astype(np.int), header=ret.header)
+        ret = Image((ret.data > mask_thresh).astype(np.int32), header=ret.header)
     return ret
 
 def reg_flirt(wsp, img, ref, initial_transform=None):
@@ -405,7 +470,8 @@ def reg_bbr(wsp):
         result = pyepi.epi_reg(wsp, wsp.reg.regfrom)
     else:
         result = epi_reg(epi=wsp.reg.regfrom, t1=wsp.structural.struc, t1brain=wsp.structural.brain, out=fsl.LOAD, wmseg=wsp.structural.wm_seg, init=wsp.reg.asl2struc, inweight=wsp.inweight, log=wsp.fsllog)
-    return result["out.nii.gz"], result["out"]
+    wsp.log.write("BBR output: %s\n" % str(result))
+    return result["out%s" % defaultExt()], result["out"]
 
     #OUTPUT
     #echo "Saving FINAL output"
